@@ -1278,12 +1278,10 @@ go_plugin_use_unref (GOPlugin *plugin)
  * go_plugin_get_dependencies_ids:
  * @plugin: #GOPlugin
  *
- * Returns: (element-type char) (transfer full): the list of identifiers of
+ * Returns: (element-type utf8) (transfer full): the list of identifiers of
  * plugins that @plugin depends on.
  * 	All these plugins will be automatically activated before activating the
- * 	@plugin itself.  The caller must free the returned list together with
- * 	the strings it points to (use g_slist_free_full (list, g_free) to do
- * 	this).
+ * 	@plugin itself.
  **/
 GSList *
 go_plugin_get_dependencies_ids (GOPlugin *plugin)
@@ -1301,7 +1299,7 @@ go_plugin_get_dependencies_ids (GOPlugin *plugin)
  * go_plugin_get_services:
  * @plugin: #GOPlugin
  *
- * Returns: (element-type GOPluginService) (transfer none):A list of services.
+ * Returns: (element-type GOPluginService) (transfer none): A list of services.
  * The list must not be freed or changed.
  **/
 GSList *
@@ -1516,33 +1514,51 @@ go_plugin_db_activate_plugin_list (GSList *plugins, GOErrorInfo **ret_error)
 void
 go_plugin_db_deactivate_plugin_list (GSList *plugins, GOErrorInfo **ret_error)
 {
-	GSList *error_list = NULL;
+	GSList *plugins_copy = g_slist_copy (plugins);
 
 	GO_INIT_RET_ERROR_INFO (ret_error);
-	GO_SLIST_FOREACH (plugins, GOPlugin, plugin,
-		GOErrorInfo *error;
 
-		go_plugin_deactivate (plugin, &error);
-		if (error != NULL) {
-			GOErrorInfo *new_error;
+	while (plugins_copy) {
+		gboolean progress = FALSE;
+		GSList *bad = NULL;
+		GSList *error_list = NULL;
 
-			new_error = go_error_info_new_printf (
-			            _("Couldn't deactivate plugin \"%s\" (ID: %s)."),
-			            plugin->name, plugin->id);
-			go_error_info_add_details (new_error, error);
-			GO_SLIST_PREPEND (error_list, new_error);
+		while (plugins_copy) {
+			GOPlugin *plugin = plugins_copy->data;
+			GOErrorInfo *error;
+
+			go_plugin_deactivate (plugin, &error);
+
+			if (error) {
+				GOErrorInfo *new_error =
+					go_error_info_new_printf (
+						_("Couldn't deactivate plugin \"%s\" (ID: %s)."),
+						plugin->name, plugin->id);
+				go_error_info_add_details (new_error, error);
+				GO_SLIST_PREPEND (error_list, new_error);
+				bad = g_slist_prepend (bad, plugin);
+			} else
+				progress = TRUE;
+
+			plugins_copy = g_slist_delete_link (plugins_copy, plugins_copy);
+		}
+
+		if (progress) {
+			g_slist_free_full (error_list, (GDestroyNotify)go_error_info_free);
+			plugins_copy = bad;
+		} else {
+			g_slist_free (bad);
+			GO_SLIST_REVERSE (error_list);
+			*ret_error = go_error_info_new_from_error_list (error_list);
+			break;
 		}
-	);
-	if (error_list != NULL) {
-		GO_SLIST_REVERSE (error_list);
-		*ret_error = go_error_info_new_from_error_list (error_list);
 	}
 }
 
 /**
  * go_plugins_get_available_plugins:
  *
- * Returns: (element-type GOPlugin) (transfer container): the list of available
+ * Returns: (element-type GOPlugin) (transfer none): the list of available
  * plugins. The returned value must not be freed and stays valid until calling
  * plugins_rescan or plugins_shutdown.
  **/
@@ -1555,8 +1571,8 @@ go_plugins_get_available_plugins (void)
 /**
  * go_plugins_get_active_plugins:
  *
- * Returns: (element-type char) (transfer container): the list of active
- * plugins names. The caller needs to free the list, but not the content.
+ * Returns: (element-type utf8) (transfer container): the list of active
+ * plugins names.
  **/
 GSList *
 go_plugins_get_active_plugins (void)
@@ -1890,6 +1906,7 @@ go_plugins_shutdown (void)
 
 	if (plugins_marked_for_deactivation_hash != NULL) {
 		g_hash_table_destroy (plugins_marked_for_deactivation_hash);
+		plugins_marked_for_deactivation_hash = NULL;
 	}
 
 	/* deactivate all plugins */
