@@ -498,6 +498,113 @@ go_gtk_editable_enters (GtkWindow *window, GtkWidget *w)
 		G_CALLBACK (cb_activate_default), window);
 }
 
+void
+go_gtk_widget_replace (GtkWidget *victim, GtkWidget *replacement)
+{
+	GtkContainer *parent = GTK_CONTAINER (gtk_widget_get_parent (victim));
+
+	if (GTK_IS_GRID (parent)) {
+		int col, row, width, height;
+		gtk_container_child_get (parent,
+					 victim,
+					 "left-attach", &col,
+					 "top-attach", &row,
+					 "width", &width,
+					 "height", &height,
+					 NULL);
+		gtk_container_remove (parent, victim);
+		gtk_grid_attach (GTK_GRID (parent), replacement,
+				 col, row, width, height);
+	} else if (GTK_IS_BOX (parent)) {
+		GtkBox *box = GTK_BOX (parent);
+		gboolean expand, fill;
+		guint padding;
+		GtkPackType pack_type;
+		int pos;
+
+		gtk_box_query_child_packing (box, victim,
+					     &expand, &fill,
+					     &padding, &pack_type);
+		gtk_container_child_get (parent, victim,
+					 "position", &pos,
+					 NULL);
+		gtk_container_remove (parent, victim);
+		gtk_container_add (parent, replacement);
+		gtk_box_set_child_packing (box, replacement,
+					   expand, fill,
+					   padding, pack_type);
+		gtk_box_reorder_child (box, replacement, pos);
+	} else {
+		g_error ("Unsupported container: %s",
+			 g_type_name_from_instance ((gpointer)parent));
+	}
+}
+
+struct go_gtk_grid_data {
+	GtkWidget *child;
+	int top_attach, left_attach, height, width;
+};
+
+static GList *
+get_grid_data (GtkGrid *grid)
+{
+	GtkContainer *cont = GTK_CONTAINER (grid);
+	GList *children = gtk_container_get_children (cont);
+	GList *p;
+
+	for (p = children; p; p = p->next) {
+		GtkWidget *child = p->data;
+		struct go_gtk_grid_data *data = g_new (struct go_gtk_grid_data, 1);
+		data->child = child;
+		gtk_container_child_get (cont, child,
+					 "top-attach", &data->top_attach,
+					 "height", &data->height,
+					 "left-attach", &data->left_attach,
+					 "width", &data->width,
+					 NULL);
+		p->data = data;
+	}
+
+	return children;
+}
+
+static int
+by_row (struct go_gtk_grid_data *a, struct go_gtk_grid_data *b)
+{
+	return a->top_attach - b->top_attach;
+}
+
+
+void
+go_gtk_grid_remove_row (GtkGrid *grid, int row)
+{
+	GtkContainer *cont = GTK_CONTAINER (grid);
+	GList *children =
+		g_list_sort (get_grid_data (grid), (GCompareFunc)by_row);
+	GList *p;
+
+	for (p = children; p; p = p->next) {
+		struct go_gtk_grid_data *data = p->data;
+
+		if (data->top_attach <= row &&
+		    data->top_attach + data->height > row)
+			data->height--;
+
+		if (data->top_attach > row)
+			data->top_attach--;
+
+		if (data->height <= 0)
+			gtk_container_remove (cont, data->child);
+		else
+			gtk_container_child_set (cont, data->child,
+						 "height", data->height,
+						 "top-attach", data->top_attach,
+						 NULL);
+	}
+	g_list_free_full (children, (GDestroyNotify)g_free);
+}
+
+
 /**
  * go_gtk_widget_disable_focus:
  * @w: #GtkWidget
@@ -513,31 +620,6 @@ go_gtk_widget_disable_focus (GtkWidget *w)
 	gtk_widget_set_can_focus (w, FALSE);
 }
 
-/**
- * go_pango_measure_string:
- * @context: #PangoContext
- * @font_desc: #PangoFontDescription
- * @str: The text to measure.
- *
- * A utility function to measure text.
- *
- * Returns: the pixel length of @str according to @context.
- **/
-int
-go_pango_measure_string (PangoContext *context, PangoFontDescription const *font_desc, char const *str)
-{
-	PangoLayout *layout = pango_layout_new (context);
-	int width;
-
-	pango_layout_set_text (layout, str, -1);
-	pango_layout_set_font_description (layout, font_desc);
-	pango_layout_get_pixel_size (layout, &width, NULL);
-
-	g_object_unref (layout);
-
-	return width;
-}
-
 static void
 cb_parent_mapped (GtkWidget *parent, GtkWindow *window)
 {
@@ -636,6 +718,7 @@ gu_delete_handler (GtkDialog *dialog,
 	return TRUE; /* Do not destroy */
 }
 
+#if !GTK_CHECK_VERSION(3,4,0)
 static void
 recent_func_log_func (G_GNUC_UNUSED const gchar   *log_domain,
 		      G_GNUC_UNUSED GLogLevelFlags log_level,
@@ -645,6 +728,7 @@ recent_func_log_func (G_GNUC_UNUSED const gchar   *log_domain,
 	/* We just want to suppress the warnings */
 	/* http://bugzilla.gnome.org/show_bug.cgi?id=664587 */
 }
+#endif
 
 /**
  * go_gtk_file_sel_dialog:
@@ -660,8 +744,6 @@ go_gtk_file_sel_dialog (GtkWindow *toplevel, GtkWidget *w)
 {
 	gboolean result = FALSE;
 	gulong delete_handler;
-	guint log_handler;
-	const char *domain = "Gtk";
 
 	g_return_val_if_fail (GTK_IS_WINDOW (toplevel), FALSE);
 	g_return_val_if_fail (GTK_IS_FILE_CHOOSER (w), FALSE);
@@ -676,13 +758,21 @@ go_gtk_file_sel_dialog (GtkWindow *toplevel, GtkWidget *w)
 	gtk_widget_show (w);
 	gtk_grab_add (w);
 
+#if GTK_CHECK_VERSION(3,4,0)
+	gtk_main ();
+#else
 	/* Hack: the gtk file dialog warns when it can't access a new file, so shut it up.  */
 	/* http://bugzilla.gnome.org/show_bug.cgi?id=664587 */
 	/* This hack should be unneccessary in GTK 3.3.5 and later  */
-	log_handler = g_log_set_handler (domain, G_LOG_LEVEL_WARNING,
-					 recent_func_log_func, NULL);
-	gtk_main ();
-	g_log_remove_handler (domain, log_handler);
+	{
+		const char *domain = "Gtk";
+		guint log_handler =
+			g_log_set_handler (domain, G_LOG_LEVEL_WARNING,
+					   recent_func_log_func, NULL);
+		gtk_main ();
+		g_log_remove_handler (domain, log_handler);
+	}
+#endif
 
 	g_signal_handler_disconnect (w, delete_handler);
 
