@@ -33,7 +33,6 @@
 
 #define DOUBLE double
 #define SUFFIX(_n) _n
-#define M_PIgo    3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117
 #define go_strto go_strtod
 #define GO_const(_n) _n
 
@@ -41,26 +40,27 @@
 #include "go-complex.c"
 #undef DOUBLE
 #undef SUFFIX
-#undef M_PIgo
 #undef go_strto
 #undef GO_const
+#define LONG_DOUBLE_VERSION
 
 #ifdef HAVE_SUNMATH_H
 #include <sunmath.h>
 #endif
 #define DOUBLE long double
 #define SUFFIX(_n) _n ## l
-#define M_PIgo    3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117L
 #define go_strto go_strtold
 #define GO_const(_n) _n ## L
 #endif
 
 #endif
 
+#define COMPLEX SUFFIX(GOComplex)
+
 /* ------------------------------------------------------------------------- */
 
 char *
-SUFFIX(go_complex_to_string) (SUFFIX(GOComplex) const *src, char const *reformat,
+SUFFIX(go_complex_to_string) (COMPLEX const *src, char const *reformat,
 		   char const *imformat, char imunit)
 {
 	char *re_buffer = NULL;
@@ -125,7 +125,7 @@ SUFFIX(is_unit_imaginary) (char const *src, DOUBLE *im, char *imunit)
 }
 
 int
-SUFFIX(go_complex_from_string) (SUFFIX(GOComplex) *dst, char const *src, char *imunit)
+SUFFIX(go_complex_from_string) (COMPLEX *dst, char const *src, char *imunit)
 {
 	DOUBLE x, y;
 	char *end;
@@ -179,7 +179,7 @@ SUFFIX(go_complex_from_string) (SUFFIX(GOComplex) *dst, char const *src, char *i
 /* ------------------------------------------------------------------------- */
 
 void
-SUFFIX(go_complex_to_polar) (DOUBLE *mod, DOUBLE *angle, SUFFIX(GOComplex) const *src)
+SUFFIX(go_complex_to_polar) (DOUBLE *mod, DOUBLE *angle, COMPLEX const *src)
 {
 	*mod = SUFFIX(go_complex_mod) (src);
 	*angle = SUFFIX(go_complex_angle) (src);
@@ -188,15 +188,25 @@ SUFFIX(go_complex_to_polar) (DOUBLE *mod, DOUBLE *angle, SUFFIX(GOComplex) const
 /* ------------------------------------------------------------------------- */
 
 void
-SUFFIX(go_complex_from_polar) (SUFFIX(GOComplex) *dst, DOUBLE mod, DOUBLE angle)
+SUFFIX(go_complex_from_polar) (COMPLEX *dst, DOUBLE mod, DOUBLE angle)
 {
-	SUFFIX(go_complex_init) (dst, mod * SUFFIX(cos) (angle), mod * SUFFIX(sin) (angle));
+	SUFFIX(go_complex_init) (dst,
+				 mod * SUFFIX(cos) (angle),
+				 mod * SUFFIX(sin) (angle));
+}
+
+static  void
+SUFFIX(go_complex_from_polar_pi) (COMPLEX *dst, DOUBLE mod, DOUBLE angle)
+{
+	SUFFIX(go_complex_init) (dst,
+				 mod * SUFFIX(go_cospi) (angle),
+				 mod * SUFFIX(go_sinpi) (angle));
 }
 
 /* ------------------------------------------------------------------------- */
 
 void
-SUFFIX(go_complex_mul) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *a, SUFFIX(GOComplex) const *b)
+SUFFIX(go_complex_mul) (COMPLEX *dst, COMPLEX const *a, COMPLEX const *b)
 {
 	SUFFIX(go_complex_init) (dst,
 		      a->re * b->re - a->im * b->im,
@@ -206,7 +216,7 @@ SUFFIX(go_complex_mul) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *a, SUFF
 /* ------------------------------------------------------------------------- */
 
 void
-SUFFIX(go_complex_div) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *a, SUFFIX(GOComplex) const *b)
+SUFFIX(go_complex_div) (COMPLEX *dst, COMPLEX const *a, COMPLEX const *b)
 {
 	DOUBLE bmod = SUFFIX(go_complex_mod) (b);
 
@@ -230,67 +240,161 @@ SUFFIX(go_complex_div) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *a, SUFF
 /* ------------------------------------------------------------------------- */
 
 void
-SUFFIX(go_complex_sqrt) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *src)
+SUFFIX(go_complex_sqrt) (COMPLEX *dst, COMPLEX const *src)
 {
-	if (SUFFIX(go_complex_real_p) (src)) {
-		if (src->re >= 0)
-			SUFFIX(go_complex_init) (dst, SUFFIX(sqrt) (src->re), 0);
-		else
-			SUFFIX(go_complex_init) (dst, 0, SUFFIX(sqrt) (-src->re));
-	} else
-		SUFFIX(go_complex_from_polar) (dst,
-				    SUFFIX(sqrt) (SUFFIX(go_complex_mod) (src)),
-				    SUFFIX(go_complex_angle) (src) / 2);
+	SUFFIX(go_complex_from_polar_pi)
+		(dst,
+		 SUFFIX(sqrt) (SUFFIX(go_complex_mod) (src)),
+		 SUFFIX(go_complex_angle_pi) (src) / 2);
 }
 
 /* ------------------------------------------------------------------------- */
 
-void
-SUFFIX(go_complex_pow) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *a, SUFFIX(GOComplex) const *b)
+/*
+ * Subtract a whole number leaving a fractional part in [-0.5,+0.5]
+ * We only look at z->h, so this probably won't do the full reduction
+ * for huge z.
+ */
+static void
+SUFFIX(reduce1) (SUFFIX(GOQuad) *z)
 {
-	if (SUFFIX(go_complex_zero_p) (a) && SUFFIX(go_complex_real_p) (b)) {
-		if (b->re <= 0)
-			SUFFIX(go_complex_invalid) (dst);
-		else
-			SUFFIX(go_complex_real) (dst, 0);
-	} else {
-		DOUBLE res_r, res_a1, res_a2, res_a2_pi, r, arg;
-		SUFFIX(GOComplex) F;
+	SUFFIX(GOQuad) d;
 
-		SUFFIX(go_complex_to_polar) (&r, &arg, a);
-		res_r = SUFFIX(pow) (r, b->re) * SUFFIX(exp) (-b->im * arg);
-		res_a1 = b->im * SUFFIX(log) (r);
-		res_a2 = b->re * arg;
-		res_a2_pi = b->re * SUFFIX(go_complex_angle_pi) (a);
+	if (SUFFIX (fabs) (z->h) <= 0.5)
+		return;
 
-		res_a2_pi = SUFFIX(fmod) (res_a2_pi, 2);
-		if (res_a2_pi < 0) res_a2_pi += 2;
+	SUFFIX(go_quad_init) (&d, SUFFIX(floor) (z->h + 0.5));
+	SUFFIX(go_quad_sub) (z, z, &d);
+}
 
-		/*
-		 * Problem: sometimes res_a2 is a nice fraction of pi.
-		 * Actually adding it will introduce pointless rounding
-		 * errors.
-		 */
-		if (res_a2_pi == 0.5) {
-			res_a2 = 0;
-			SUFFIX(go_complex_init) (&F, 0, 1);
-		} else if (res_a2_pi == 1) {
-			res_a2 = 0;
-			SUFFIX(go_complex_real) (&F, -1);
-		} else if (res_a2_pi == 1.5) {
-			res_a2 = 0;
-			SUFFIX(go_complex_init) (&F, 0, -1);
-		} else
-			SUFFIX(go_complex_real) (&F, 1);
+static void
+SUFFIX(mulmod1) (SUFFIX(GOQuad) *dst, SUFFIX(GOQuad) const *qa_, DOUBLE b)
+{
+	SUFFIX(GOQuad) qa = *qa_, qfb, qfa, qp, res;
+	DOUBLE wb, wa;
+	int ea, eb, de;
 
-		SUFFIX(go_complex_from_polar) (dst, res_r, res_a1 + res_a2);
-		SUFFIX(go_complex_mul) (dst, dst, &F);
+	(void)SUFFIX(frexp) (SUFFIX(go_quad_value) (&qa), &ea);
+	(void)SUFFIX(frexp) (b, &eb);
+	if (ea + eb <= 0) {
+		/* |ab| <= 2 */
+		SUFFIX(go_quad_init) (&qfb, b);
+		SUFFIX(go_quad_mul) (dst, &qfb, &qa);
+		return;
+	}
+
+	de = (ea - eb) / 2;
+	if (de) {
+		DOUBLE f = SUFFIX(ldexp) (1, de);
+		b *= f;
+		qa.h /= f;
+		qa.l /= f;
+	}
+
+	wb = SUFFIX(floor) (b + 0.5);
+	b -= wb;
+	SUFFIX(go_quad_init) (&qfb, b);
+
+	wa = SUFFIX (floor) (SUFFIX(go_quad_value) (&qa) + 0.5);
+	SUFFIX(go_quad_init) (&qfa, wa);
+	SUFFIX(go_quad_sub) (&qfa, &qa, &qfa);
+
+	/*
+	 * Compute (wb+qfb)*(wa+qfa) mod 1.
+	 *
+	 * We can ignore wa*wb.  (And qfb==b.)
+	 */
+
+	SUFFIX(go_quad_mul) (&res, &qfa, &qfb);
+
+	SUFFIX(go_quad_mul12) (&qp, wa, b);
+	SUFFIX(reduce1) (&qp);
+	SUFFIX(go_quad_add) (&res, &res, &qp);
+
+	SUFFIX(go_quad_init) (&qp, wb);
+	SUFFIX(go_quad_mul) (&qp, &qp, &qfa);
+	SUFFIX(reduce1) (&qp);
+	SUFFIX(go_quad_add) (&res, &res, &qp);
+
+	SUFFIX(reduce1) (&res);
+
+	*dst = res;
+}
+
+void
+SUFFIX(go_complex_pow) (COMPLEX *dst, COMPLEX const *a, COMPLEX const *b)
+{
+	if (b->im == 0) {
+		if (SUFFIX(go_complex_real_p) (a) && a->re >= 0) {
+			SUFFIX(go_complex_init) (dst, pow (a->re, b->re), 0);
+			return;
+		}
+
+		if (b->re == 0) {
+			SUFFIX(go_complex_init) (dst, 1, 0);
+			return;
+		}
+
+		if (b->re == 1) {
+			*dst = *a;
+			return;
+		}
+
+		if (b->re == 2) {
+			SUFFIX(go_complex_mul) (dst, a, a);
+			return;
+		}
+	}
+
+	{
+		DOUBLE e1, e2;
+		int e;
+		SUFFIX(GOQuad) qr, qa, qb, qarg, qrr, qra;
+		void *state = SUFFIX(go_quad_start) ();
+
+		/* Convert a to polar coordinates.  */
+		SUFFIX(go_quad_init) (&qa, a->re);
+		SUFFIX(go_quad_init) (&qb, a->im);
+		SUFFIX(go_quad_atan2pi) (&qarg, &qb, &qa);
+		SUFFIX(go_quad_hypot) (&qr, &qa, &qb);
+
+		/* Compute result modulus.  */
+		SUFFIX(go_quad_init) (&qa, b->re);
+		SUFFIX(go_quad_pow) (&qa, &e1, &qr, &qa);
+		SUFFIX(go_quad_init) (&qb, -b->im);
+		SUFFIX(go_quad_mul) (&qb, &qb, &qarg);
+		SUFFIX(go_quad_mul) (&qb, &qb, &SUFFIX(go_quad_pi));
+		SUFFIX(go_quad_exp) (&qb, &e2, &qb);
+		SUFFIX(go_quad_mul) (&qrr, &qa, &qb);
+		e = CLAMP (e1 + e2, G_MININT, G_MAXINT);
+		qrr.h = SUFFIX(ldexp) (qrr.h, e);
+		qrr.l = SUFFIX(ldexp) (qrr.l, e);
+
+		/* Compute result angle.  */
+		SUFFIX(go_quad_log) (&qa, &qr);
+		SUFFIX(go_quad_div) (&qa, &qa, &SUFFIX(go_quad_2pi));
+		SUFFIX(mulmod1) (&qa, &qa, b->im);
+		SUFFIX(mulmod1) (&qb, &qarg, b->re / 2);
+		SUFFIX(go_quad_add) (&qa, &qa, &qb);
+		SUFFIX(go_quad_add) (&qra, &qa, &qa);
+
+		/* Convert to rectangular coordinates.  */
+		SUFFIX(go_quad_sinpi) (&qa, &qra);
+		SUFFIX(go_quad_mul) (&qa, &qa, &qrr);
+		SUFFIX(go_quad_cospi) (&qb, &qra);
+		SUFFIX(go_quad_mul) (&qb, &qb, &qrr);
+
+		SUFFIX(go_complex_init) (dst,
+					 SUFFIX(go_quad_value) (&qb),
+					 SUFFIX(go_quad_value) (&qa));
+
+		SUFFIX(go_quad_end) (state);
 	}
 }
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_init) (SUFFIX(GOComplex) *dst, DOUBLE re, DOUBLE im)
+void SUFFIX(go_complex_init) (COMPLEX *dst, DOUBLE re, DOUBLE im)
 {
 	dst->re = re;
 	dst->im = im;
@@ -298,13 +402,13 @@ void SUFFIX(go_complex_init) (SUFFIX(GOComplex) *dst, DOUBLE re, DOUBLE im)
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_invalid) (SUFFIX(GOComplex) *dst)
+void SUFFIX(go_complex_invalid) (COMPLEX *dst)
 {
 	dst->re = SUFFIX(go_nan);
 	dst->im = SUFFIX(go_nan);
 }
 
-void SUFFIX(go_complex_real) (SUFFIX(GOComplex) *dst, DOUBLE re)
+void SUFFIX(go_complex_real) (COMPLEX *dst, DOUBLE re)
 {
 	dst->re = re;
 	dst->im = 0;
@@ -312,35 +416,35 @@ void SUFFIX(go_complex_real) (SUFFIX(GOComplex) *dst, DOUBLE re)
 
 /* ------------------------------------------------------------------------- */
 
-int SUFFIX(go_complex_real_p) (SUFFIX(GOComplex) const *src)
+int SUFFIX(go_complex_real_p) (COMPLEX const *src)
 {
 	return src->im == 0;
 }
 
 /* ------------------------------------------------------------------------- */
 
-int SUFFIX(go_complex_zero_p) (SUFFIX(GOComplex) const *src)
+int SUFFIX(go_complex_zero_p) (COMPLEX const *src)
 {
 	return src->re == 0 && src->im == 0;
 }
 
 /* ------------------------------------------------------------------------- */
 
-int SUFFIX(go_complex_invalid_p) (SUFFIX(GOComplex) const *src)
+int SUFFIX(go_complex_invalid_p) (COMPLEX const *src)
 {
 	return !(SUFFIX(go_finite) (src->re) && SUFFIX(go_finite) (src->im));
 }
 
 /* ------------------------------------------------------------------------- */
 
-DOUBLE SUFFIX(go_complex_mod) (SUFFIX(GOComplex) const *src)
+DOUBLE SUFFIX(go_complex_mod) (COMPLEX const *src)
 {
 	return SUFFIX(hypot) (src->re, src->im);
 }
 
 /* ------------------------------------------------------------------------- */
 
-DOUBLE SUFFIX(go_complex_angle) (SUFFIX(GOComplex) const *src)
+DOUBLE SUFFIX(go_complex_angle) (COMPLEX const *src)
 {
 	return SUFFIX(atan2) (src->im, src->re);
 }
@@ -351,30 +455,21 @@ DOUBLE SUFFIX(go_complex_angle) (SUFFIX(GOComplex) const *src)
  * nice round numbers not suffering from rounding errors).
  */
 
-DOUBLE SUFFIX(go_complex_angle_pi) (SUFFIX(GOComplex) const *src)
+DOUBLE SUFFIX(go_complex_angle_pi) (COMPLEX const *src)
 {
-	if (src->im == 0)
-		return (src->re >= 0 ? 0 : -1);
-
-	if (src->re == 0)
-		return (src->im >= 0 ? 0.5 : -0.5);
-
-	/* We could do quarters too */
-
-	/* Fallback.  */
-	return SUFFIX(go_complex_angle) (src) / M_PIgo;
+	return SUFFIX(go_atan2pi) (src->im, src->re);
 }
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_conj) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *src)
+void SUFFIX(go_complex_conj) (COMPLEX *dst, COMPLEX const *src)
 {
 	SUFFIX(go_complex_init) (dst, src->re, -src->im);
 }
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_scale_real) (SUFFIX(GOComplex) *dst, DOUBLE f)
+void SUFFIX(go_complex_scale_real) (COMPLEX *dst, DOUBLE f)
 {
 	dst->re *= f;
 	dst->im *= f;
@@ -382,30 +477,28 @@ void SUFFIX(go_complex_scale_real) (SUFFIX(GOComplex) *dst, DOUBLE f)
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_add) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *a, SUFFIX(GOComplex) const *b)
+void SUFFIX(go_complex_add) (COMPLEX *dst, COMPLEX const *a, COMPLEX const *b)
 {
 	SUFFIX(go_complex_init) (dst, a->re + b->re, a->im + b->im);
 }
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_sub) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *a, SUFFIX(GOComplex) const *b)
+void SUFFIX(go_complex_sub) (COMPLEX *dst, COMPLEX const *a, COMPLEX const *b)
 {
 	SUFFIX(go_complex_init) (dst, a->re - b->re, a->im - b->im);
 }
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_exp) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *src)
+void SUFFIX(go_complex_exp) (COMPLEX *dst, COMPLEX const *src)
 {
-	SUFFIX(go_complex_init) (dst,
-		SUFFIX(exp) (src->re) * SUFFIX(cos) (src->im),
-		SUFFIX(exp) (src->re) * SUFFIX(sin) (src->im));
+	SUFFIX(go_complex_from_polar) (dst, SUFFIX(exp) (src->re), src->im);
 }
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_ln) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *src)
+void SUFFIX(go_complex_ln) (COMPLEX *dst, COMPLEX const *src)
 {
 	SUFFIX(go_complex_init) (dst,
 		SUFFIX(log) (SUFFIX(go_complex_mod) (src)),
@@ -414,7 +507,7 @@ void SUFFIX(go_complex_ln) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *src
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_sin) ( SUFFIX(GOComplex) *dst,  SUFFIX(GOComplex) const *src)
+void SUFFIX(go_complex_sin) ( COMPLEX *dst,  COMPLEX const *src)
 {
 	SUFFIX(go_complex_init) (dst,
 		SUFFIX(sin) (src->re) * SUFFIX(cosh) (src->im),
@@ -423,7 +516,7 @@ void SUFFIX(go_complex_sin) ( SUFFIX(GOComplex) *dst,  SUFFIX(GOComplex) const *
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_cos) (SUFFIX(GOComplex) *dst,  SUFFIX(GOComplex) const *src)
+void SUFFIX(go_complex_cos) (COMPLEX *dst,  COMPLEX const *src)
 {
 	SUFFIX(go_complex_init) (dst,
 		SUFFIX(cos) (src->re) * SUFFIX(cosh) (src->im),
@@ -432,11 +525,13 @@ void SUFFIX(go_complex_cos) (SUFFIX(GOComplex) *dst,  SUFFIX(GOComplex) const *s
 
 /* ------------------------------------------------------------------------- */
 
-void SUFFIX(go_complex_tan) (SUFFIX(GOComplex) *dst, SUFFIX(GOComplex) const *src)
+void SUFFIX(go_complex_tan) (COMPLEX *dst, COMPLEX const *src)
 {
-	SUFFIX(GOComplex) s, c;
+	COMPLEX s, c;
 
 	SUFFIX(go_complex_sin) (&s, src);
 	SUFFIX(go_complex_cos) (&c, src);
 	SUFFIX(go_complex_div) (dst, &s, &c);
 }
+
+/* ------------------------------------------------------------------------- */
