@@ -29,6 +29,7 @@
 #include <locale.h>
 #include <signal.h>
 #include <errno.h>
+#include <string.h>
 
 double go_nan;
 double go_pinf;
@@ -68,7 +69,7 @@ running_under_buggy_valgrind (void)
 void
 _go_math_init (void)
 {
-	const char *bug_url = "http://bugzilla.gnome.org/enter_bug.cgi?product=libgoffice";
+	const char *bug_url = "https://bugzilla.gnome.org/enter_bug.cgi?product=libgoffice";
 	char *old_locale;
 	double d;
 #ifdef SIGFPE
@@ -268,7 +269,18 @@ go_fake_ceil (double x)
 double
 go_fake_round (double x)
 {
-	double y = go_fake_floor (fabs (x) + 0.5);
+	double y;
+
+	x = go_d2d (x);
+
+	if (x == floor (x))
+		return x;
+
+	/*
+	 * Adding a half here is ok.  The only problematic non-integer
+	 * case is nextafter(0.5,-1) for which we want to produce 1 here.
+	 */
+	y = go_fake_floor (fabs (x) + 0.5);
 	return (x < 0) ? -y : y;
 }
 
@@ -288,8 +300,12 @@ go_fake_trunc (double x)
 double
 go_rint (double x)
 {
+#ifdef HAVE_RINT
+	return rint (x);
+#else
 	double y = floor (fabs (x) + 0.5);
 	return (x < 0) ? -y : y;
+#endif
 }
 
 int
@@ -700,7 +716,16 @@ go_fake_ceill (long double x)
 long double
 go_fake_roundl (long double x)
 {
-	long double y = go_fake_floorl (fabsl (x) + 0.5L);
+	long double y;
+
+	if (x == floorl (x))
+		return x;
+
+	/*
+	 * Adding a half here is ok.  The only problematic non-integer
+	 * case is nextafter(0.5,-1) for which we want to produce 1 here.
+	 */
+	y = go_fake_floorl (fabsl (x) + 0.5L);
 	return (x < 0) ? -y : y;
 }
 
